--
-- ReversingCamera
-- You can look back throw the left window when you are reversing with truck and trailer.
-- @author:    	kenny456 (kenny456@seznam.cz)
-- @history:	v1.0 - 2017-09-11 -
--				v1.1 - 2017-09-15 - added controls for position of reverse camera
-- 
ReversingCamera = {};
local ReversingCamera_directory = g_currentModDirectory;

function ReversingCamera.prerequisitesPresent(specializations)
    return true;
end;

function ReversingCamera:load(xmlFile)
	for i=1, self.numCameras do
        if self.cameras[i].isInside then
           self.internalCamera = self.cameras[i].cameraNode
        end;
    end;
	xt,yt,zt = getTranslation(self.internalCamera);
    xr,yr,zr = getRotation(self.internalCamera);
    self.camPositionX = xt;  
    self.camPositionY = yt; 
    self.camPositionZ = zt;
	self.tx=xt;
    self.ty=yt;
    self.tz=zt;
	self.reverseCamera = false;
	self.revCamShowHelp = false;
	self.transXmax = 0.6;
	self.transZmax = 0.1;
	self.rotYstart = 4.2;
end;
function ReversingCamera:delete()
end;

function ReversingCamera:readStream(streamId, connection)
end;

function ReversingCamera:writeStream(streamId, connection)
end;

function ReversingCamera:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ReversingCamera:keyEvent(unicode, sym, modifier, isDown)
end;

function ReversingCamera:update(dt)
	if self:getIsActive() and self.cameras[self.camIndex].isRotatable and self.cameras[self.camIndex].isInside == true then
		local math_pi_double  = math.pi*2
		self.rozdilY = self.cameras[self.camIndex].rotY - self.rotYstart;
		if(math_pi_double > self.cameras[self.camIndex].rotY and self.cameras[self.camIndex].rotY > (self.rotYstart)) then
			self.reverseCamera = true;
			self.camPositionX = (self.tx + (self.rozdilY * 0.5 * self.transXmax / 0.6));
			self.camPositionZ = (self.tz + (self.rozdilY * 0.5 * self.transZmax / 0.1));
			if self.camPositionX >= self.tx + self.transXmax then
				self.camPositionX = self.tx + self.transXmax;
			end;
			if self.camPositionZ >= self.tz + self.transZmax then
				self.camPositionZ = self.tz + self.transZmax;
			end;
			if self.cameras[self.camIndex].rotY >= 6 then
				self.cameras[self.camIndex].rotY = 6;
			end;
			if InputBinding.isPressed(InputBinding.revCamOut) then	
				if self.transXmax <= self.tx + 2 then
					self.camPositionX = self.camPositionX + 0.01;
					self.transXmax = self.transXmax + 0.01;
				end;
			end;
			if InputBinding.isPressed(InputBinding.revCamIn) then
				if self.transXmax >= 0.1 then
					self.camPositionX = self.camPositionX - 0.01;
					self.transXmax = self.transXmax - 0.01;
				end;
			end;
			if InputBinding.isPressed(InputBinding.revCamForward) then
				if self.transZmax <= 1.5 then
					self.camPositionZ = self.camPositionZ + 0.01;
					self.transZmax = self.transZmax + 0.01;
				end;
			end;
			if InputBinding.isPressed(InputBinding.revCamBack) then
				if self.transZmax >= 0.01 then
					self.camPositionZ = self.camPositionZ - 0.01;
					self.transZmax = self.transZmax - 0.01;
				end;
			end;
			if InputBinding.hasEvent(InputBinding.revCamReset) then
				self.transXmax = 0.6;
				self.transZmax = 0.1;
			end;
			if InputBinding.hasEvent(InputBinding.revCamHelp) then
				self.revCamShowHelp = not self.revCamShowHelp;
			end;
		else
			self.camPositionX = self.tx;
			self.camPositionZ = self.tz;
			self.reverseCamera = false;
		end;
		if self.internalCamera ~= nil and self.cameras[self.camIndex].isInside == true then
			setTranslation(self.internalCamera, self.camPositionX, self.camPositionY, self.camPositionZ);
		end;
	end;
end;

function ReversingCamera:updateTick(dt)
end;

function ReversingCamera:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local transXmax = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#RevCamTransXmax"), self.transXmax);
	local transZmax = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#RevCamTransZmax"), self.transZmax);
	self.transXmax = transXmax;
	self.transZmax = transZmax;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function ReversingCamera:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'RevCamTransXmax="' .. tostring(self.transXmax) ..'" RevCamTransZmax="' .. tostring(self.transZmax) ..'"';
	return attributes, nil;
end;

function ReversingCamera:draw()
	if self:getIsActive() and self.reverseCamera == true then
		if self.revCamShowHelp == true then
			g_currentMission:addHelpButtonText(g_i18n:getText("revCamHelpHide"),InputBinding.revCamHelp);
			g_currentMission:addHelpButtonText(g_i18n:getText('revCamOut'),InputBinding.revCamOut);
			g_currentMission:addHelpButtonText(g_i18n:getText("revCamIn"),InputBinding.revCamIn);
			g_currentMission:addHelpButtonText(g_i18n:getText("revCamForward"),InputBinding.revCamForward);
			g_currentMission:addHelpButtonText(g_i18n:getText("revCamBack"),InputBinding.revCamBack);
			g_currentMission:addHelpButtonText(g_i18n:getText("revCamReset"),InputBinding.revCamReset);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("revCamHelpShow"),InputBinding.revCamHelp);
		end;
	end;
end;

function ReversingCamera:onLeave()
end;